import os
from typing import List

from deciphon_sched.cffi import ffi, lib
from deciphon_sched.error import SchedError, SchedWrapperError
from deciphon_sched.rc import RC

__all__ = ["sched_init", "sched_cleanup", "sched_wipe", "sched_health_check"]


def sched_init(file_name: str):
    rc = RC(lib.sched_init(file_name.encode()))
    rc.raise_for_status()


def sched_cleanup():
    rc = RC(lib.sched_cleanup())
    rc.raise_for_status()


def sched_health_check(file) -> List[str]:
    fd = os.dup(file.fileno())
    if fd == -1:
        raise SchedWrapperError(RC.SCHED_FAIL_OPEN_FILE)

    fp = lib.fdopen(fd, b"r+")
    if fp == ffi.NULL:
        raise SchedWrapperError(RC.SCHED_FAIL_OPEN_FILE)

    ptr = ffi.new("struct sched_health *")
    if ptr == ffi.NULL:
        lib.fclose(fp)
        raise SchedError(RC.SCHED_NOT_ENOUGH_MEMORY)

    ptr[0].fp = fp
    ptr[0].num_errors = 0
    rc = RC(lib.sched_health_check(ptr))

    lib.fclose(fp)
    rc.raise_for_status()

    file.flush()
    file.seek(0)

    errors: List[str] = []
    for row in file:
        errors.append(row.strip())

    return errors


def sched_wipe():
    rc = RC(lib.sched_wipe())
    rc.raise_for_status()



import os
import shutil
import subprocess
import tarfile
import urllib.request
from pathlib import Path

PWD = Path(os.path.dirname(os.path.abspath(__file__)))
USER = "EBI-Metagenomics"
PROJECT = "deciphon-sched"
VERSION = "0.5.0"
CMAKE_OPTS = [
    "-DCMAKE_BUILD_TYPE=Release",
    "-DENABLE_ALL_WARNINGS=ON",
    "-DCMAKE_POSITION_INDEPENDENT_CODE=ON",
    "-DSCHED_BUILD_TESTS=OFF",
]


def rm(folder: Path, pattern: str):
    for filename in folder.glob(pattern):
        filename.unlink()


def get_cmake_bin():
    from cmake import CMAKE_BIN_DIR

    bins = [str(v) for v in Path(CMAKE_BIN_DIR).glob("cmake*")]
    return str(sorted(bins, key=lambda v: len(v))[0])


def cleanup_intree_artifacts():
    rm(PWD / "deciphon_sched", "cffi.c")
    rm(PWD / "deciphon_sched", "*.o")
    rm(PWD / "deciphon_sched", "*.so")
    rm(PWD / "deciphon_sched", "*.dylib")


def build_deps():
    ext_dir = PWD / ".ext_deps"
    shutil.rmtree(ext_dir, ignore_errors=True)

    prj_dir = ext_dir / f"{PROJECT}-{VERSION}"
    build_dir = prj_dir / "build"
    os.makedirs(build_dir, exist_ok=True)

    url = f"https://github.com/{USER}/{PROJECT}/archive/refs/tags/v{VERSION}.tar.gz"

    with urllib.request.urlopen(url) as rf:
        data = rf.read()

    tar_filename = f"{PROJECT}-{VERSION}.tar.gz"

    with open(ext_dir / tar_filename, "wb") as lf:
        lf.write(data)

    with tarfile.open(ext_dir / tar_filename) as tf:
        tf.extractall(ext_dir)

    cmake_bin = get_cmake_bin()
    subprocess.check_call(
        [cmake_bin, "-S", str(prj_dir), "-B", str(build_dir)] + CMAKE_OPTS
    )
    subprocess.check_call([cmake_bin, "--build", str(build_dir), "--config", "Release"])
    subprocess.check_call(
        [cmake_bin, "--install", str(build_dir), "--prefix", str(ext_dir)]
    )
    rm(ext_dir / "lib", "*.dylib")
    rm(ext_dir / "lib", "*.so*")
    rm(ext_dir / "lib64", "*.dylib")
    rm(ext_dir / "lib64", "*.so*")


if __name__ == "__main__":
    from cffi import FFI

    ffibuilder = FFI()

    cleanup_intree_artifacts()
    build_deps()
    library_dirs = [PWD / ".ext_deps" / "lib", PWD / ".ext_deps" / "lib64"]
    include_dirs = [PWD / ".ext_deps" / "include"]

    with open(PWD / "deciphon_sched" / "interface.h", "r") as f:
        interface_h = f.read()

    ffibuilder.cdef(interface_h)
    ffibuilder.set_source(
        "deciphon_sched.cffi",
        """
        #include "sched/sched.h"
        """,
        language="c",
        libraries=["sched"],
        library_dirs=[str(d) for d in library_dirs if d.exists()],
        include_dirs=[str(d) for d in include_dirs if d.exists()],
    )
    ffibuilder.compile(verbose=True)



from deciphon_sched.cffi import ffi, lib
from deciphon_sched.rc import RC

__all__ = ["SchedError", "SchedWrapperError"]


def sched_error_string(rc: RC) -> str:
    return ffi.string(lib.sched_error_string(rc.value))


class SchedError(Exception):
    def __init__(self, rc: RC):
        self.rc: RC = rc
        self.msg: str = sched_error_string(rc)


class SchedWrapperError(SchedError):
    def __init__(self, rc: RC):
        self.rc: RC = rc
        self.msg: str = sched_error_string(rc)



from dataclasses import dataclass
from typing import Any, List

from deciphon_sched.cffi import ffi, lib
from deciphon_sched.error import SchedError
from deciphon_sched.prod import sched_prod, sched_prod_new
from deciphon_sched.rc import RC
from deciphon_sched.seq import sched_seq, sched_seq_new

__all__ = [
    "sched_scan",
    "sched_scan_get_by_id",
    "sched_scan_get_by_job_id",
    "sched_scan_get_seqs",
    "sched_scan_get_prods",
    "sched_scan_get_all",
    "sched_scan_new",
    "sched_scan_add_seq",
]


@dataclass
class sched_scan:
    id: int
    db_id: int

    multi_hits: bool
    hmmer3_compat: bool

    job_id: int

    ptr: Any

    def refresh(self):
        c = self.ptr[0]

        self.id = int(c.id)
        self.db_id = int(c.db_id)

        self.multi_hits = bool(c.multi_hits)
        self.hmmer3_compat = bool(c.hmmer3_compat)

        self.job_id = int(c.job_id)


def possess(ptr):
    c = ptr[0]
    return sched_scan(
        int(c.id),
        int(c.db_id),
        bool(c.multi_hits),
        bool(c.hmmer3_compat),
        int(c.job_id),
        ptr,
    )


def new_scan():
    ptr = ffi.new("struct sched_scan *")
    if ptr == ffi.NULL:
        raise SchedError(RC.SCHED_NOT_ENOUGH_MEMORY)
    return ptr


def sched_scan_add_seq(name: str, data: str):
    lib.sched_scan_add_seq(name.encode(), data.encode())


def sched_scan_new(db_id: int, multi_hits: bool, hmmer3_compat: bool) -> sched_scan:
    ptr = new_scan()
    lib.sched_scan_init(ptr, db_id, multi_hits, hmmer3_compat)
    return possess(ptr)


def sched_scan_get_by_id(scan_id: int) -> sched_scan:
    ptr = new_scan()
    rc = RC(lib.sched_scan_get_by_id(ptr, scan_id))
    rc.raise_for_status()
    return possess(ptr)


def sched_scan_get_by_job_id(job_id: int) -> sched_scan:
    ptr = new_scan()
    rc = RC(lib.sched_scan_get_by_job_id(ptr, job_id))
    rc.raise_for_status()
    return possess(ptr)


def sched_scan_get_seqs(scan_id: int):
    seqs: List[sched_seq] = []
    ptr = sched_seq_new(0, scan_id).ptr
    rc = RC(lib.sched_scan_get_seqs(scan_id, lib.append_seq, ptr, ffi.new_handle(seqs)))
    rc.raise_for_status()
    return seqs


def sched_scan_get_prods(scan_id: int) -> List[sched_prod]:
    prods: List[sched_prod] = []
    ptr = sched_prod_new().ptr
    hdl = ffi.new_handle(prods)
    rc = RC(lib.sched_scan_get_prods(scan_id, lib.append_prod, ptr, hdl))
    rc.raise_for_status()
    return prods


def sched_scan_get_all() -> List[sched_scan]:
    scans: List[sched_scan] = []
    ptr = new_scan()
    rc = RC(lib.sched_scan_get_all(lib.append_scan, ptr, ffi.new_handle(scans)))
    rc.raise_for_status()
    return scans


@ffi.def_extern()
def append_scan(ptr, arg):
    sched_scans = ffi.from_handle(arg)
    sched_scans.append(possess(ptr))



from dataclasses import dataclass
from typing import Any, List

from deciphon_sched.cffi import ffi, lib
from deciphon_sched.error import SchedError
from deciphon_sched.rc import RC

__all__ = [
    "sched_hmm",
    "sched_hmm_new",
    "sched_hmm_get_all",
    "sched_hmm_get_by_filename",
    "sched_hmm_get_by_id",
    "sched_hmm_get_by_job_id",
    "sched_hmm_get_by_xxh3",
    "sched_hmm_remove",
]


@dataclass
class sched_hmm:
    id: int
    xxh3: int
    filename: str
    job_id: int
    ptr: Any

    def refresh(self):
        c = self.ptr[0]

        self.id = int(c.id)
        self.xxh3 = int(c.xxh3)

        self.filename = ffi.string(c.filename)
        self.job_id = int(c.job_id)


def string(cdata) -> str:
    return ffi.string(cdata).decode()


def possess(ptr) -> sched_hmm:
    c = ptr[0]
    return sched_hmm(int(c.id), int(c.xxh3), string(c.filename), int(c.job_id), ptr)


def new_hmm():
    ptr = ffi.new("struct sched_hmm *")
    if ptr == ffi.NULL:
        raise SchedError(RC.SCHED_NOT_ENOUGH_MEMORY)
    lib.sched_hmm_init(ptr)
    return ptr


def sched_hmm_new(filename: str) -> sched_hmm:
    ptr = new_hmm()
    rc = RC(lib.sched_hmm_set_file(ptr, filename.encode()))
    rc.raise_for_status()
    return possess(ptr)


def sched_hmm_get_by_id(hmm_id: int) -> sched_hmm:
    ptr = new_hmm()
    rc = RC(lib.sched_hmm_get_by_id(ptr, hmm_id))
    rc.raise_for_status()
    return possess(ptr)


def sched_hmm_get_by_job_id(hmm_id: int) -> sched_hmm:
    ptr = new_hmm()
    rc = RC(lib.sched_hmm_get_by_job_id(ptr, hmm_id))
    rc.raise_for_status()
    return possess(ptr)


def sched_hmm_get_by_xxh3(xxh3: int) -> sched_hmm:
    ptr = new_hmm()
    rc = RC(lib.sched_hmm_get_by_xxh3(ptr, xxh3))
    rc.raise_for_status()
    return possess(ptr)


def sched_hmm_get_by_filename(filename: str) -> sched_hmm:
    ptr = new_hmm()
    rc = RC(lib.sched_hmm_get_by_filename(ptr, filename.encode()))
    rc.raise_for_status()
    return possess(ptr)


def sched_hmm_get_all() -> List[sched_hmm]:
    hmms: List[sched_hmm] = []
    ptr = new_hmm()
    rc = RC(lib.sched_hmm_get_all(lib.append_hmm, ptr, ffi.new_handle(hmms)))
    rc.raise_for_status()
    return hmms


def sched_hmm_remove(hmm_id: int):
    rc = RC(lib.sched_hmm_remove(hmm_id))
    rc.raise_for_status()


@ffi.def_extern()
def append_hmm(ptr, arg):
    sched_hmms = ffi.from_handle(arg)
    sched_hmms.append(possess(ptr))



from dataclasses import dataclass
from typing import Any, List, Optional

from deciphon_sched.cffi import ffi, lib
from deciphon_sched.error import SchedError
from deciphon_sched.rc import RC

__all__ = [
    "sched_seq",
    "sched_seq_new",
    "sched_seq_get_by_id",
    "sched_seq_get_all",
    "sched_seq_scan_next",
]


def string(cdata) -> str:
    return ffi.string(cdata).decode()


@dataclass
class sched_seq:
    id: int
    scan_id: int
    name: str
    data: str
    ptr: Any

    def refresh(self):
        c = self.ptr[0]

        self.id = int(c.id)
        self.scan_id = int(c.scan_id)

        self.name = string(c.name)
        self.data = string(c.data)


def possess(ptr):
    c = ptr[0]
    return sched_seq(
        int(c.id),
        int(c.scan_id),
        string(c.name),
        string(c.data),
        ptr,
    )


def sched_seq_new(seq_id: int, scan_id: int) -> sched_seq:
    ptr = ffi.new("struct sched_seq *")
    if ptr == ffi.NULL:
        raise SchedError(RC.SCHED_NOT_ENOUGH_MEMORY)
    lib.sched_seq_init(ptr, seq_id, scan_id, "".encode(), "".encode())
    return possess(ptr)


def new_seq():
    return sched_seq_new(0, 0).ptr


def sched_seq_get_by_id(seq_id: int) -> sched_seq:
    ptr = new_seq()
    rc = RC(lib.sched_seq_get_by_id(ptr, seq_id))
    rc.raise_for_status()
    return possess(ptr)


def sched_seq_get_all() -> List[sched_seq]:
    seqs: List[sched_seq] = []
    ptr = new_seq()
    rc = RC(lib.sched_seq_get_all(lib.append_seq, ptr, ffi.new_handle(seqs)))
    rc.raise_for_status()
    return seqs


def sched_seq_scan_next(seq: sched_seq) -> Optional[sched_seq]:
    rc = RC(lib.sched_seq_scan_next(seq.ptr))
    if rc == RC.SCHED_SEQ_NOT_FOUND:
        return None
    rc.raise_for_status()
    seq.refresh()
    return seq


@ffi.def_extern()
def append_seq(ptr, arg):
    sched_seqs = ffi.from_handle(arg)
    sched_seqs.append(possess(ptr))



from dataclasses import dataclass
from typing import Any, List

from deciphon_sched.cffi import ffi, lib
from deciphon_sched.error import SchedError
from deciphon_sched.rc import RC

__all__ = [
    "sched_db_add",
    "sched_db_get_all",
    "sched_db_get_by_filename",
    "sched_db_get_by_hmm_id",
    "sched_db_get_by_id",
    "sched_db_get_by_xxh3",
    "sched_db_remove",
]


@dataclass
class sched_db:
    id: int
    xxh3: int
    filename: str
    hmm_id: int
    ptr: Any


def string(cdata) -> str:
    return ffi.string(cdata).decode()


def possess(ptr):
    c = ptr[0]
    return sched_db(int(c.id), int(c.xxh3), string(c.filename), int(c.hmm_id), ptr)


def new_db():
    ptr = ffi.new("struct sched_db *")
    if ptr == ffi.NULL:
        raise SchedError(RC.SCHED_NOT_ENOUGH_MEMORY)
    return ptr


def sched_db_add(filename: str) -> sched_db:
    ptr = new_db()
    rc = RC(lib.sched_db_add(ptr, filename.encode()))
    rc.raise_for_status()
    return possess(ptr)


def sched_db_remove(db_id: int):
    rc = RC(lib.sched_db_remove(db_id))
    rc.raise_for_status()


def sched_db_get_by_id(db_id: int) -> sched_db:
    ptr = new_db()
    rc = RC(lib.sched_db_get_by_id(ptr, db_id))
    rc.raise_for_status()
    return possess(ptr)


def sched_db_get_by_xxh3(xxh3: int) -> sched_db:
    ptr = new_db()
    rc = RC(lib.sched_db_get_by_xxh3(ptr, xxh3))
    rc.raise_for_status()
    return possess(ptr)


def sched_db_get_by_filename(filename: str) -> sched_db:
    ptr = new_db()
    rc = RC(lib.sched_db_get_by_filename(ptr, filename.encode()))
    rc.raise_for_status()
    return possess(ptr)


def sched_db_get_by_hmm_id(hmm_id: int) -> sched_db:
    ptr = new_db()
    rc = RC(lib.sched_db_get_by_hmm_id(ptr, hmm_id))
    rc.raise_for_status()
    return possess(ptr)


def sched_db_get_all() -> List[sched_db]:
    dbs: List[sched_db] = []
    ptr = new_db()
    rc = RC(lib.sched_db_get_all(lib.append_db, ptr, ffi.new_handle(dbs)))
    rc.raise_for_status()
    return dbs


@ffi.def_extern()
def append_db(ptr, arg):
    sched_dbs = ffi.from_handle(arg)
    sched_dbs.append(possess(ptr))



from dataclasses import dataclass
from enum import Enum
from typing import Any, List, Optional, Union

from deciphon_sched.cffi import ffi, lib
from deciphon_sched.error import SchedError
from deciphon_sched.hmm import sched_hmm
from deciphon_sched.rc import RC
from deciphon_sched.scan import sched_scan

__all__ = [
    "sched_job",
    "sched_job_get_by_id",
    "sched_job_get_all",
    "sched_job_set_run",
    "sched_job_set_fail",
    "sched_job_set_done",
    "sched_job_submit",
    "sched_job_increment_progress",
    "sched_job_remove",
]


class sched_job_type(int, Enum):
    SCHED_SCAN = 0
    SCHED_HMM = 1


class sched_job_state(str, Enum):
    SCHED_PEND = "pend"
    SCHED_RUN = "run"
    SCHED_DONE = "done"
    SCHED_FAIL = "fail"


def string(cdata) -> str:
    return ffi.string(cdata).decode()


@dataclass
class sched_job:
    id: int
    type: sched_job_type

    state: sched_job_state
    progress: int
    error: str

    submission: int
    exec_started: int
    exec_ended: int

    ptr: Any

    def refresh(self):
        c = self.ptr[0]

        self.id = int(c.id)
        self.type = sched_job_type(int(c.type))

        self.state = sched_job_state(string(c.state))
        self.progress = int(c.progress)
        self.error = ffi.string(c.error).decode()

        self.submission = int(c.submission)
        self.exec_started = int(c.exec_started)
        self.exec_ended = int(c.exec_ended)


def possess(ptr) -> sched_job:
    c = ptr[0]
    return sched_job(
        int(c.id),
        sched_job_type(int(c.type)),
        sched_job_state(string(c.state)),
        int(c.progress),
        ffi.string(c.error).decode(),
        int(c.submission),
        int(c.exec_started),
        int(c.exec_ended),
        ptr,
    )


def new_job(job_type: sched_job_type = sched_job_type.SCHED_SCAN):
    ptr = ffi.new("struct sched_job *")
    if ptr == ffi.NULL:
        raise SchedError(RC.SCHED_NOT_ENOUGH_MEMORY)
    lib.sched_job_init(ptr, job_type.value)
    return ptr


def sched_job_new(job_type: sched_job_type) -> sched_job:
    return possess(new_job(job_type))


def sched_job_get_by_id(job_id: int) -> sched_job:
    ptr = new_job()
    rc = RC(lib.sched_job_get_by_id(ptr, job_id))
    rc.raise_for_status()
    return possess(ptr)


def sched_job_next_pend() -> Optional[sched_job]:
    ptr = new_job()
    rc = RC(lib.sched_job_next_pend(ptr))
    if rc == RC.SCHED_JOB_NOT_FOUND:
        return None
    rc.raise_for_status()
    return possess(ptr)


def sched_job_get_all() -> List[sched_job]:
    jobs: List[sched_job] = []
    ptr = new_job()
    rc = RC(lib.sched_job_get_all(lib.append_job, ptr, ffi.new_handle(jobs)))
    rc.raise_for_status()
    return jobs


def sched_job_set_run(job_id: int):
    rc = RC(lib.sched_job_set_run(job_id))
    rc.raise_for_status()


def sched_job_set_fail(job_id: int, msg: str):
    rc = RC(lib.sched_job_set_fail(job_id, msg.encode()))
    rc.raise_for_status()


def sched_job_set_done(job_id: int):
    rc = RC(lib.sched_job_set_done(job_id))
    rc.raise_for_status()


def sched_job_submit(actual_job: Union[sched_hmm, sched_scan]) -> sched_job:
    if isinstance(actual_job, sched_hmm):
        ptr = sched_job_new(sched_job_type.SCHED_HMM).ptr
    else:
        assert isinstance(actual_job, sched_scan)
        ptr = sched_job_new(sched_job_type.SCHED_SCAN).ptr

    rc = RC(lib.sched_job_submit(ptr, actual_job.ptr))
    rc.raise_for_status()

    actual_job.refresh()
    return possess(ptr)


def sched_job_increment_progress(job_id: int, progress: int):
    rc = RC(lib.sched_job_increment_progress(job_id, progress))
    rc.raise_for_status()


def sched_job_remove(job_id: int):
    rc = RC(lib.sched_job_remove(job_id))
    rc.raise_for_status()


@ffi.def_extern()
def append_job(ptr, arg):
    sched_jobs = ffi.from_handle(arg)
    sched_jobs.append(possess(ptr))



import os
from dataclasses import dataclass
from typing import Any, List

from deciphon_sched.cffi import ffi, lib
from deciphon_sched.error import SchedError
from deciphon_sched.rc import RC

__all__ = [
    "sched_prod",
    "sched_prod_new",
    "sched_prod_get_by_id",
    "sched_prod_get_all",
    "sched_prod_add_file",
]


@dataclass
class sched_prod:
    id: int

    scan_id: int
    seq_id: int

    profile_name: str
    abc_name: str

    alt_loglik: float
    null_loglik: float

    profile_typeid: str
    version: str

    match: str

    ptr: Any


def string(cdata) -> str:
    return ffi.string(cdata).decode()


def possess(ptr):
    c = ptr[0]
    return sched_prod(
        int(c.id),
        int(c.scan_id),
        int(c.seq_id),
        string(c.profile_name),
        string(c.abc_name),
        float(c.alt_loglik),
        float(c.null_loglik),
        string(c.profile_typeid),
        string(c.version),
        string(c.match),
        ptr,
    )


def new_prod():
    ptr = ffi.new("struct sched_prod *")
    if ptr == ffi.NULL:
        raise SchedError(RC.SCHED_NOT_ENOUGH_MEMORY)
    return ptr


def sched_prod_new() -> sched_prod:
    return possess(new_prod())


def sched_prod_get_by_id(prod_id: int) -> sched_prod:
    ptr = new_prod()
    rc = RC(lib.sched_prod_get_by_id(ptr, prod_id))
    rc.raise_for_status()
    return possess(ptr)


def sched_prod_get_all() -> List[sched_prod]:
    prods: List[sched_prod] = []
    ptr = new_prod()
    rc = RC(lib.sched_prod_get_all(lib.append_prod, ptr, ffi.new_handle(prods)))
    rc.raise_for_status()
    return prods


def sched_prod_add_file(filename: str):
    rc = RC(lib.sched_prod_add_file(filename.encode()))
    rc.raise_for_status()


@ffi.def_extern()
def append_prod(ptr, arg):
    sched_prods = ffi.from_handle(arg)
    sched_prods.append(possess(ptr))



from __future__ import annotations

from enum import Enum

__all__ = ["RC"]


class RC(int, Enum):
    SCHED_OK = 0
    SCHED_END = 1
    SCHED_HMM_NOT_FOUND = 2
    SCHED_SCAN_NOT_FOUND = 3
    SCHED_DB_NOT_FOUND = 4
    SCHED_JOB_NOT_FOUND = 5
    SCHED_PROD_NOT_FOUND = 6
    SCHED_SEQ_NOT_FOUND = 7
    SCHED_NOT_ENOUGH_MEMORY = 8
    SCHED_FAIL_PARSE_FILE = 9
    SCHED_FAIL_OPEN_FILE = 10
    SCHED_FAIL_CLOSE_FILE = 11
    SCHED_FAIL_WRITE_FILE = 12
    SCHED_FAIL_READ_FILE = 13
    SCHED_FAIL_REMOVE_FILE = 14
    SCHED_INVALID_FILE_NAME = 15
    SCHED_INVALID_FILE_NAME_EXT = 16
    SCHED_TOO_SHORT_FILE_NAME = 17
    SCHED_TOO_LONG_FILE_NAME = 18
    SCHED_TOO_LONG_FILE_PATH = 19
    SCHED_FILE_NAME_NOT_SET = 20
    SCHED_HMM_ALREADY_EXISTS = 21
    SCHED_DB_ALREADY_EXISTS = 22
    SCHED_ASSOC_HMM_NOT_FOUND = 23
    SCHED_FAIL_BIND_STMT = 24
    SCHED_FAIL_EVAL_STMT = 25
    SCHED_FAIL_GET_FRESH_STMT = 26
    SCHED_FAIL_GET_COLUMN_TEXT = 27
    SCHED_FAIL_EXEC_STMT = 28
    SCHED_FAIL_PREPARE_STMT = 29
    SCHED_FAIL_RESET_STMT = 30
    SCHED_FAIL_OPEN_SCHED_FILE = 31
    SCHED_FAIL_CLOSE_SCHED_FILE = 32
    SCHED_SQLITE8_NOT_THREAD_SAFE = 33
    SCHED_SQLITE8_TOO_OLD = 34
    SCHED_FAIL_BEGIN_TRANSACTION = 35
    SCHED_FAIL_END_TRANSACTION = 36
    SCHED_FAIL_ROLLBACK_TRANSACTION = 37

    def raise_for_status(self):
        from deciphon_sched.error import SchedError

        if self.value != RC.SCHED_OK and self.value != RC.SCHED_END:
            raise SchedError(self)
